<?php

declare(strict_types=1);

namespace Paysera\Action;

defined('ABSPATH') || exit;

use Paysera\Admin\PayseraDeliveryAdmin;
use Paysera\Entity\PayseraDeliverySettings;
use Paysera\Helper\PayseraDeliveryLibraryHelper;
use WC_Cache_Helper;

class PayseraDeliveryActions
{
    public function build(): void
    {
        add_action('admin_post_paysera_delivery_gateway_change', [$this, 'changeDeliveryGatewayStatus']);
        add_action('add_option', [$this, 'validateSettingsBeforeCreate'], 10, 2);
        add_action('update_option', [$this, 'validateSettingsBeforeUpdate'], 10, 3);
        add_filter('removable_query_args', [$this, 'filterRemovableQueryArgs']);
    }

    public function changeDeliveryGatewayStatus(): void
    {
        $this->updateDeliveryGatewayStatus(
            sanitize_text_field($_GET['gateway']),
            sanitize_text_field($_GET['change']) === 'enable'
        );

        wp_redirect(
            'admin.php?page=paysera-delivery&tab=' . PayseraDeliveryAdmin::TAB_DELIVERY_GATEWAYS_LIST_SETTINGS
        );
    }

    /**
     * @param array $deliveryGateways
     */
    public function setDeliveryGatewayTitles(array $deliveryGateways): void
    {
        foreach ($deliveryGateways as $deliveryGateway) {
            $this->updateDeliveryGatewayTitle($deliveryGateway->getCode(), $deliveryGateway->getDescription());
        }
    }

    /**
     * @param array $deliveryGateways
     */
    public function reSyncDeliveryGatewayStatus(array $deliveryGateways): void
    {
        foreach ($deliveryGateways as $deliveryGateway) {
            if ($deliveryGateway->isEnabled() === false) {
                $this->updateDeliveryGatewayStatus($deliveryGateway->getCode(), false);
            }
        }
    }

    /**
     * @param array $shipmentMethods
     */
    public function syncShipmentMethodsStatus(array $shipmentMethods): void
    {
        foreach ($shipmentMethods as $shipmentMethod) {
            $this->updateShipmentMethodStatus($shipmentMethod->getCode(), $shipmentMethod->isEnabled());
        }
    }

    public function updateDeliveryGatewaySetting(
        string $deliveryGatewayCode,
        string $settingName,
        $settingValue
    ): void {
        $this->updateOptions($deliveryGatewayCode . '_' . $settingName, $settingValue);
    }

    public function updateResolvedProjectId(string $projectId): void
    {
        $this->updateOptions(PayseraDeliverySettings::RESOLVED_PROJECT_ID, $projectId);
    }

    private function updateOptions(string $optionName, $optionValue): void
    {
        $options = get_option(PayseraDeliverySettings::DELIVERY_GATEWAYS_SETTINGS_NAME);

        $options[$optionName] = $optionValue;

        update_option(PayseraDeliverySettings::DELIVERY_GATEWAYS_SETTINGS_NAME, $options);
    }

    private function updateDeliveryGatewayStatus(string $deliveryGateway, bool $isEnabled): void
    {
        $options = get_option(PayseraDeliverySettings::DELIVERY_GATEWAYS_SETTINGS_NAME);

        $options[PayseraDeliverySettings::DELIVERY_GATEWAYS][$deliveryGateway] = $isEnabled;

        WC_Cache_Helper::get_transient_version('shipping', true);

        update_option(PayseraDeliverySettings::DELIVERY_GATEWAYS_SETTINGS_NAME, $options);
    }

    private function updateDeliveryGatewayTitle(string $deliveryGateway, string $title): void
    {
        $options = get_option(PayseraDeliverySettings::DELIVERY_GATEWAYS_TITLES);

        $options[PayseraDeliverySettings::DELIVERY_GATEWAYS][$deliveryGateway] = $title;

        update_option(PayseraDeliverySettings::DELIVERY_GATEWAYS_TITLES, $options);
    }

    private function updateShipmentMethodStatus(string $shipmentMethod, bool $isEnabled): void
    {
        $options = get_option(PayseraDeliverySettings::DELIVERY_GATEWAYS_SETTINGS_NAME);

        $options[PayseraDeliverySettings::SHIPMENT_METHODS][$shipmentMethod] = $isEnabled;

        update_option(PayseraDeliverySettings::DELIVERY_GATEWAYS_SETTINGS_NAME, $options);
    }

    public function filterRemovableQueryArgs(array $removableQueryArgs): array
    {
        $removableQueryArgs[] = 'invalid-credentials';

        return $removableQueryArgs;
    }

    public function validateSettingsBeforeCreate(string $option, $value): void
    {
        $this->checkInvalidCredential($option, $value);
    }

    public function validateSettingsBeforeUpdate(string $option, $oldValue, $value): void
    {
        $this->checkInvalidCredential($option, $value);
    }

    private function checkInvalidCredential(string $option, $value): void
    {
        if ($option !== PayseraDeliverySettings::SETTINGS_NAME) {
            return;
        }

        $merchantClient = null;
        $projectId = $value['project_id'] ?? null;
        $projectPassword = $value['project_password'] ?? null;

        if ($projectId && $projectPassword) {
            $merchantClient = (new PayseraDeliveryLibraryHelper())
                ->getMerchantClientByMacCredential((int) $projectId, $projectPassword)
            ;
        }

        if ($merchantClient === null) {
            wp_safe_redirect('admin.php?page=paysera-delivery&invalid-credentials=yes');
            exit();
        }
    }
}
